@@ -197,8 +197,33 @@ var LibraryEmVal = {
     return newer(handle, argTypes, args);
   },
 
+#if NO_DYNAMIC_EXECUTION
+  $emval_get_global: function() {
+    function testGlobal(obj) {
+      obj['$$$embind_global$$$'] = obj;
+      var success = typeof $$$embind_global$$$ === 'object' && obj['$$$embind_global$$$'] === obj;
+      if (!success) {
+	delete obj['$$$embind_global$$$'];
+      }
+      return success;
+    }
+    if (typeof $$$embind_global$$$ === 'object') {
+      return $$$embind_global$$$;
+    }
+    if (typeof global === 'object' && testGlobal(global)) {
+      $$$embind_global$$$ = global;
+    } else if (typeof window === 'object' && testGlobal(window)) {
+      $$$embind_global$$$ = window;
+    }
+    if (typeof $$$embind_global$$$ === 'object') {
+      return $$$embind_global$$$;
+    }
+    throw Error('unable to get global object.');
+  },
+#else
   // appease jshint (technically this code uses eval)
   $emval_get_global: function() { return (function(){return Function;})()('return this')(); },
+#endif
   _emval_get_global__deps: ['_emval_register', '$getStringOrSymbol', '$emval_get_global'],
   _emval_get_global: function(name) {
     if(name===0){
@@ -240,18 +265,32 @@ var LibraryEmVal = {
     return returnType['toWireType'](destructors, handle);
   },
 
-  _emval_equals__deps: ['_emval_register', '$requireHandle'],
-  _emval_equals: function(first, second ){
+  _emval_equals__deps: ['$requireHandle'],
+  _emval_equals: function(first, second ) {
     first = requireHandle(first);
     second = requireHandle(second);
-    return first==second;
+    return first == second;
   },
 
-  _emval_strictly_equals__deps: ['_emval_register', '$requireHandle'],
-  _emval_strictly_equals: function(first, second ){
+  _emval_strictly_equals__deps: ['$requireHandle'],
+  _emval_strictly_equals: function(first, second ) {
     first = requireHandle(first);
     second = requireHandle(second);
-    return first===second;
+    return first === second;
+  },
+
+  _emval_greater_than__deps: ['$requireHandle'],
+  _emval_greater_than: function(first, second ) {
+    first = requireHandle(first);
+    second = requireHandle(second);
+    return first > second;
+  },
+
+  _emval_less_than__deps: ['$requireHandle'],
+  _emval_less_than: function(first, second ) {
+    first = requireHandle(first);
+    second = requireHandle(second);
+    return first < second;
   },
 
   _emval_call__deps: ['_emval_lookupTypes', '_emval_register', '$requireHandle'],
@@ -387,6 +426,28 @@ var LibraryEmVal = {
     handle = requireHandle(handle);
     return __emval_register(typeof handle);
   },
+
+  _emval_instanceof__deps: ['$requireHandle'],
+  _emval_instanceof: function(object, constructor) {
+    object = requireHandle(object);
+    constructor = requireHandle(constructor);
+    return object instanceof constructor;
+  },
+  
+  _emval_in__deps: ['$requireHandle'],
+  _emval_in: function(item, object) {
+    item = requireHandle(item);
+    object = requireHandle(object);
+    return item in object;
+  },
+
+  _emval_delete__deps: ['$requireHandle'],
+  _emval_delete: function(object, property) {
+    object = requireHandle(object);
+    property = requireHandle(property);
+    return delete object[property];
+  },
+
 };
 
 mergeInto(LibraryManager.library, LibraryEmVal);
