@@ -8,6 +8,9 @@ mergeInto(LibraryManager.library, {
 #endif
 #if __EMSCRIPTEN_HAS_workerfs_js__
     '$WORKERFS',
+#endif
+#if __EMSCRIPTEN_HAS_noderawfs_js__
+    '$NODERAWFS',
 #endif
     'stdin', 'stdout', 'stderr'],
   $FS__postset: 'FS.staticInit();' +
@@ -19,7 +22,7 @@ mergeInto(LibraryManager.library, {
   $FS: {
     root: null,
     mounts: [],
-    devices: [null],
+    devices: {},
     streams: [],
     nextInode: 1,
     nameTable: null,
@@ -1054,7 +1057,7 @@ mergeInto(LibraryManager.library, {
         if (!FS.readFiles) FS.readFiles = {};
         if (!(path in FS.readFiles)) {
           FS.readFiles[path] = 1;
-          Module['printErr']('read file: ' + path);
+          err('read file: ' + path);
         }
       }
       try {
@@ -1074,6 +1077,9 @@ mergeInto(LibraryManager.library, {
       return stream;
     },
     close: function(stream) {
+      if (FS.isClosed(stream)) {
+        throw new FS.ErrnoError(ERRNO_CODES.EBADF);
+      }
       if (stream.getdents) stream.getdents = null; // free readdir state
       try {
         if (stream.stream_ops.close) {
@@ -1084,8 +1090,15 @@ mergeInto(LibraryManager.library, {
       } finally {
         FS.closeStream(stream.fd);
       }
+      stream.fd = null;
+    },
+    isClosed: function(stream) {
+      return stream.fd === null;
     },
     llseek: function(stream, offset, whence) {
+      if (FS.isClosed(stream)) {
+        throw new FS.ErrnoError(ERRNO_CODES.EBADF);
+      }
       if (!stream.seekable || !stream.stream_ops.llseek) {
         throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
       }
@@ -1097,6 +1110,9 @@ mergeInto(LibraryManager.library, {
       if (length < 0 || position < 0) {
         throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
       }
+      if (FS.isClosed(stream)) {
+        throw new FS.ErrnoError(ERRNO_CODES.EBADF);
+      }
       if ((stream.flags & {{{ cDefine('O_ACCMODE') }}}) === {{{ cDefine('O_WRONLY')}}}) {
         throw new FS.ErrnoError(ERRNO_CODES.EBADF);
       }
@@ -1106,10 +1122,9 @@ mergeInto(LibraryManager.library, {
       if (!stream.stream_ops.read) {
         throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
       }
-      var seeking = true;
-      if (typeof position === 'undefined') {
+      var seeking = typeof position !== 'undefined';
+      if (!seeking) {
         position = stream.position;
-        seeking = false;
       } else if (!stream.seekable) {
         throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
       }
@@ -1121,6 +1136,9 @@ mergeInto(LibraryManager.library, {
       if (length < 0 || position < 0) {
         throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
       }
+      if (FS.isClosed(stream)) {
+        throw new FS.ErrnoError(ERRNO_CODES.EBADF);
+      }
       if ((stream.flags & {{{ cDefine('O_ACCMODE') }}}) === {{{ cDefine('O_RDONLY')}}}) {
         throw new FS.ErrnoError(ERRNO_CODES.EBADF);
       }
@@ -1134,10 +1152,9 @@ mergeInto(LibraryManager.library, {
         // seek to the end before writing in append mode
         FS.llseek(stream, 0, {{{ cDefine('SEEK_END') }}});
       }
-      var seeking = true;
-      if (typeof position === 'undefined') {
+      var seeking = typeof position !== 'undefined';
+      if (!seeking) {
         position = stream.position;
-        seeking = false;
       } else if (!stream.seekable) {
         throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
       }
@@ -1151,6 +1168,9 @@ mergeInto(LibraryManager.library, {
       return bytesWritten;
     },
     allocate: function(stream, offset, length) {
+      if (FS.isClosed(stream)) {
+        throw new FS.ErrnoError(ERRNO_CODES.EBADF);
+      }
       if (offset < 0 || length <= 0) {
         throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
       }
@@ -1214,17 +1234,15 @@ mergeInto(LibraryManager.library, {
     writeFile: function(path, data, opts) {
       opts = opts || {};
       opts.flags = opts.flags || 'w';
-      opts.encoding = opts.encoding || 'utf8';
-      if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
-        throw new Error('Invalid encoding type "' + opts.encoding + '"');
-      }
       var stream = FS.open(path, opts.flags, opts.mode);
-      if (opts.encoding === 'utf8') {
+      if (typeof data === 'string') {
         var buf = new Uint8Array(lengthBytesUTF8(data)+1);
         var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
-        FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
-      } else if (opts.encoding === 'binary') {
-        FS.write(stream, data, 0, data.length, 0, opts.canOwn);
+        FS.write(stream, buf, 0, actualNumBytes, undefined, opts.canOwn);
+      } else if (ArrayBuffer.isView(data)) {
+        FS.write(stream, data, 0, data.byteLength, undefined, opts.canOwn);
+      } else {
+        throw new Error('Unsupported data type');
       }
       FS.close(stream);
     },
@@ -1277,8 +1295,10 @@ mergeInto(LibraryManager.library, {
         var randomBuffer = new Uint8Array(1);
         random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
       } else if (ENVIRONMENT_IS_NODE) {
+#if ENVIRONMENT_MAY_BE_NODE
         // for nodejs
         random_device = function() { return require('crypto')['randomBytes'](1)[0]; };
+#endif // ENVIRONMENT_MAY_BE_NODE
       } else {
         // default for ES5 platforms
         random_device = function() { return (Math.random()*256)|0; };
@@ -1354,7 +1374,7 @@ mergeInto(LibraryManager.library, {
     ensureErrnoError: function() {
       if (FS.ErrnoError) return;
       FS.ErrnoError = function ErrnoError(errno, node) {
-        //Module.printErr(stackTrace()); // useful for debugging
+        //err(stackTrace()); // useful for debugging
         this.node = node;
         this.setErrno = function(errno) {
           this.errno = errno;
@@ -1924,4 +1944,3 @@ mergeInto(LibraryManager.library, {
 if (FORCE_FILESYSTEM) {
   DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.push('$FS');
 }
-
