@@ -1,19 +1,24 @@
-'''
-Removes timestamp and line info from a webgl log
-'''
+"""Removes timestamp and line info from a webgl log
+"""
 
 from __future__ import print_function
-import os, sys, re
+import os
+import re
+import sys
 
 __rootpath__ = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+
+
 def path_from_root(*pathelems):
   return os.path.join(__rootpath__, *pathelems)
 
+
 def nice(x):
-  return '0x' + ('0' * (len(x)-6)) + x[2:].upper()
+  return '0x' + ('0' * (len(x) - 6)) + x[2:].upper()
+
 
 repdata = open(path_from_root('system', 'include', 'GL', 'gl.h')).readlines() + ['\n'] + \
-          open(path_from_root('system', 'include', 'GL', 'glext.h')).readlines()
+    open(path_from_root('system', 'include', 'GL', 'glext.h')).readlines()
 reps = {}
 for rep in repdata:
   rep = rep.replace('\t', ' ').replace('\n', '')
@@ -29,7 +34,3 @@ for line in lines:
   line = line.split(' @ ')[0]
   line = re.sub('(0x[\dabcdef]+)', lambda hexx: reps[nice(hexx.group(0))] if nice(hexx.group(0)) in reps else nice(hexx.group(0)), line)
   print(line)
-
-#for i in range(100):
-#  print
-
